/********************************************************/
/* Copyright 1999, Chengjie Tu & Trac Duy Tran          */
/* All rights reserved                                  */
/* File: "huffman.h"                                    */
/* Description: Huffman codec class                     */
/* Affiliation: Johns Hopkins University, Baltimore, MD */
/*              FastVideo LLC, Columbia, MD             */
/********************************************************/

#ifndef HUFFMAN_H
#define HUFFMAN_H

class Huffman
{
 public:
  //constructor,use codebook size and symbol&frequency table to initialize
  Huffman(unsigned,int*,unsigned*);
  ~Huffman(void);                  //Destructor

#ifdef ENCODER
  bool CodeSymbol(int,class OutBitStream&); //code a symbol
  void ShowCodeBook(class ostream&);        //show codebook
#else
  int DecodeSymbol(class InBitStream&);     //decode a symbol
#endif

 private:
  struct Node        //codetree node
  {
    int symbol;      //symbol
    unsigned count;  //frequency
    Node* left;      //left child
    Node* right;     //right child
    Node* next;      //next node or parent

    Node(int s,unsigned c,Node* l,Node* r,Node* n)   //constructor
      :symbol(s),count(c),left(l),right(r),next(n){}
  }* _root;    //codetree root

#ifdef ENCODER
  struct BitList  //bit list for huffman code
  {
    bool bit;     //'0' or '1'
    BitList* next; //next bit 
    BitList(bool b):bit(b),next(0){} //constructor
  }** _code_book;  //codebook
#endif

  unsigned _size;  //codebook size
  int* _symbol;    //symbol table
  unsigned* _frequency;  //frequency table
  
  void _delete_tree(Node*);  //clear up codetree

#ifdef ENCODER
  void _code_tree(Node*);    //get codebook from codetree
#endif
};

#endif
